import VChip from 'vuetify/es5/components/VChip';
import * as VTabs from 'vuetify/es5/components/VTabs';
import * as VList from 'vuetify/es5/components/VList';
import API from './api';

export default {
  name: 'Epitech',
  props: ['settings'],
  components: {
    VChip,
    ...VTabs,
    ...VList,
    Timeline: () => import(/* webpackMode: "lazy" */'./timeline').then(_ => _.default),
  },
  data() {
    return {
      is_logged: true,
      gpa_precision: {
        loading: false,
        val: null,
      },
      user: {
        loading: true,
      },
      projects: {
        data: [],
        loading: true,
      },
      rooms: {
        data: [],
        loading: true,
      },
      upcomings: {
        data: [],
        loading: true,
      },
    };
  },
  methods: {
    getUser() {
      return API.getUser().then((user) => {
        this.is_logged = true;
        this.user = user;
      }).catch((err) => {
        this.is_logged = false;
        throw err;
      }).finally(() => {
        this.user.loading = false;
      });
    },
    getProjects() {
      return API.getCurrentProjects()
        .then((projects) => {
          this.projects.data = projects;
        }).catch((err) => {
          this.is_logged = false;
          throw err;
        })
        .finally(() => {
          this.projects.loading = false;
        });
    },
    getRoom(planning) {
      this.rooms.data = API.getRoom(planning);
      this.rooms.loading = false;
    },
    getUpcoming(planning) {
      this.upcomings.data = planning
        .filter(f => f.event_registered && f.start > new Date())
        .sort((a, b) => a.start - b.start);
      this.upcomings.loading = false;
    },
    getGpa() {
      this.gpa_precision.loading = true;
      return API.getGPAPrecision(this.user)
        .then((gpa) => {
          this.gpa_precision.val = gpa;
        })
        .finally(() => {
          this.gpa_precision.loading = false;
        });
    },
  },
  mounted() {
    if (this.VALID_CACHE) {
      this.$emit('init', true);
      return;
    }
    Promise.all([this.getUser(), this.getProjects()])
      .then(() => API.getPlanning(this.user))
      .then((planning) => {
        this.getRoom(planning);
        this.getUpcoming(planning);
      })
      .then(() => this.$emit('init', ['gpa_precision'].reduce((obj, key) => {
        const { [key]: _, ...tmp } = obj;
        return tmp;
      }, this.$data)))
      .catch(err => this.$emit('init', err));
  },

};
